from bokeh.core.properties import Int
from bokeh.models import CustomJSHover

from bokehmol.utils import Hook


class BaseFormatter(Hook, CustomJSHover):
    __implementation__ = "base_formatter.ts"

    width = Int(default=160, help="Image width")
    height = Int(default=120, help="Image height")



from bokeh.core.properties import Any, Bool, Dict, String

from bokehmol.config import settings
from bokehmol.models.base_formatter import BaseFormatter


class RDKitFormatter(BaseFormatter):
    __implementation__ = "rdkit_formatter.ts"
    __javascript__ = settings.rdkitjs_src

    remove_hs = Bool(default=True)
    sanitize = Bool(default=True)
    kekulize = Bool(default=True)
    prefer_coordgen = Bool(default=True)
    draw_options = Dict(
        String,
        Any,
        default={},
        help="https://www.rdkitjs.com/#drawing-molecules-all-options",
    )


if __name__ == "__main__":
    from bokeh.models import ColumnDataSource, HoverTool
    from bokeh.plotting import figure, show

    source = ColumnDataSource(
        data=dict(x=[1, 8, 3], y=[2, 5, 7], SMILES=["CCO", "c1ccccc1", "[n]"])
    )

    plot = figure(
        x_range=(0, 10),
        y_range=(0, 10),
        title="RDKit Hover",
        background_fill_color="#efefef",
        tools="",
    )

    hover_tooltip = """
    <div style="font-size: 14px;">
        <div style="position: relative; left=0;">@SMILES{custom}</div>
        <div style="position: relative;">
            <span style="font-size: 10px; word-wrap: break-word;">@SMILES</span>
        </div>
    </div>"""

    plot.add_tools(
        HoverTool(
            tooltips=hover_tooltip,
            formatters={
                "@SMILES": RDKitFormatter(
                    sanitize=False,
                    draw_options={
                        "addAtomIndices": True,
                    },
                )
            },
        )
    )

    plot.circle("x", "y", size=10, line_width=0, fill_color="red", source=source)

    show(plot)



import typing as t

Tooltips = t.Optional[t.Union[str, t.List[t.Tuple[str, str]]]]


def register_alias(rdkit_hover: bool = True, smiles_hover: bool = True) -> None:
    """Register the alias for hover tools to be used directly in the `tools`
    parameter of a figure, i.e. `rdkit_hover` and `smiles_hover`.
    """
    if rdkit_hover:
        from bokehmol.models.rdkit_hover import RDKitHover

        RDKitHover.register_alias("rdkit_hover", lambda: RDKitHover())
    if smiles_hover:
        from bokehmol.models.smilesdrawer_hover import SmilesDrawerHover

        SmilesDrawerHover.register_alias("smiles_hover", lambda: SmilesDrawerHover())


def rdkit(
    smiles_column: str = "SMILES",
    tooltips: Tooltips = None,
    width: int = 160,
    height: int = 120,
    remove_hs: bool = True,
    sanitize: bool = True,
    kekulize: bool = True,
    prefer_coordgen: bool = True,
    draw_options: t.Optional[t.Dict[str, t.Any]] = None,
    **kwargs: t.Any,
) -> "RDKitHover":
    """Hover tool that uses RDKit.js and the RDKit's Minimal Lib to depict
    SMILES strings on hover.

    Notes
    -----
    See https://www.npmjs.com/package/@rdkit/rdkit for the readme.

    Parameters
    ----------
    smiles_column: str = "SMILES"
        Column in the `ColumnDataSource` object containing the SMILES string.
    tooltips: t.Union[str, t.List[t.Tuple[str, str]]] = None
        Tooltips to render below the depiction. Can be an HTML string, or a list of
        tuples specifying the display name and column name that you want to display.
        Column names must be prefixed with `@` (or `$` for bokeh's internal
        variables), see bokeh's docs for more info on tooltips formats::

            >>> bokehmol.hover.rdkit(
            ...     tooltips=[
            ...         ("SMILES", "@SMILES"),
            ...         ("Name", "@{Molecule name}"),
            ...     ]
            ... )

    width: int = 160
        Image width in pixels
    height: int = 120
        Image height in pixels
    remove_hs: bool = True
        Remove hydrogens from the depiction
    sanitize: bool = True
        Sanitize the molecule
    kekulize: bool = True
        Kekulize the molecule
    prefer_coordgen: bool = True
        Prefer the CoordGen library for macrocycle rendering
    draw_options: t.Dict[str, t.Any] = None
        RDKit's MolDrawOptions to control the style of the drawing:
        https://www.rdkitjs.com/#drawing-molecules-all-options.::

            >>> bokehmol.hover.rdkit(
            ...     draw_options={
            ...         "addAtomIndices": True,
            ...     }
            ... )

    **kwargs: t.Any
        Additional parameters passed to bokeh's Hover tool.

    Returns
    -------
    An RDKit-based hover tool ready to be added to a bokeh plot::

        >>> from bokeh.plotting import figure
        >>> plot = figure(...)
        >>> hover_tool = bokehmol.hover.rdkit()
        >>> plot.add_tools(hover_tool)

    """
    from bokehmol.models.rdkit_hover import RDKitHover

    if tooltips is None:
        tooltips = []
    if draw_options is None:
        draw_options = {}
    return RDKitHover(
        smiles_column=smiles_column,
        tooltips=tooltips,
        width=width,
        height=height,
        remove_hs=remove_hs,
        sanitize=sanitize,
        kekulize=kekulize,
        prefer_coordgen=prefer_coordgen,
        draw_options=draw_options,
        **kwargs,
    )


def smiles_drawer(
    smiles_column: str = "SMILES",
    tooltips: Tooltips = None,
    width: int = 160,
    height: int = 120,
    theme: t.Literal[
        "light",
        "dark",
        "oldschool",
        "solarized",
        "solarized-dark",
        "matrix",
        "github",
        "carbon",
        "cyberpunk",
        "gruvbox",
        "gruvbox-dark",
    ] = "light",
    background_colour: str = "transparent",
    mol_options: t.Optional[t.Dict[str, t.Any]] = None,
    reaction_options: t.Optional[t.Dict[str, t.Any]] = None,
    **kwargs: t.Any,
) -> "SmilesDrawerHover":
    """Hover tool that uses SmilesDrawer to depict SMILES strings on hover.

    Notes
    -----
    See https://github.com/reymond-group/smilesDrawer#readme for the readme.
    Please cite the SmilesDrawer paper doi:10.1021/acs.jcim.7b00425 if you use this
    in academic work.

    Parameters
    ----------
    smiles_column: str = "SMILES"
        Column in the `ColumnDataSource` object containing the SMILES string.
    tooltips: t.Union[str, t.List[t.Tuple[str, str]]] = None
        Tooltips to render below the depiction. Can be an HTML string, or a list of
        tuples specifying the display name and column name that you want to display.
        Column names must be prefixed with `@` (or `$` for bokeh's internal
        variables), see bokeh's docs for more info on tooltips formats::

            >>> bokehmol.hover.smiles_drawer(
            ...     tooltips=[
            ...         ("SMILES", "@SMILES"),
            ...         ("Name", "@{Molecule name}"),
            ...     ]
            ... )

    width: int = 160
        Image width in pixels
    height: int = 120
        Image height in pixels
    theme: str = "light"
        Theme used for the rendering. One of the following list: light, dark,
        oldschool, solarized, solarized-dark, matrix, github, carbon, cyberpunk,
        gruvbox, gruvbox-dark.
    background_colour: str = "transparent"
        Any valid CSS color specification.
    mol_options: t.Dict[str, t.Any] = None
        SmilesDrawer options to control the style of the molecule's drawing:
        https://smilesdrawer.surge.sh/playground.html.::

            >>> bokehmol.hover.smiles_drawer(
            ...     mol_options={
            ...         "atomVisualization": "balls",
            ...     }
            ... )

    reaction_options: t.Dict[str, t.Any] = None
        Same as above for reaction's drawing.
    **kwargs: t.Any
        Additional parameters passed to bokeh's Hover tool.

    Returns
    -------
    A SmilesDrawer-based hover tool ready to be added to a bokeh plot::

        >>> from bokeh.plotting import figure
        >>> plot = figure(...)
        >>> hover_tool = bokehmol.hover.smiles_drawer()
        >>> plot.add_tools(hover_tool)

    """
    from bokehmol.models.smilesdrawer_hover import SmilesDrawerHover

    if tooltips is None:
        tooltips = []
    if mol_options is None:
        mol_options = {}
    if reaction_options is None:
        reaction_options = {}
    return SmilesDrawerHover(
        smiles_column=smiles_column,
        tooltips=tooltips,
        width=width,
        height=height,
        theme=theme,
        background_colour=background_colour,
        mol_options=mol_options,
        reaction_options=reaction_options,
        **kwargs,
    )



from bokeh.core.properties import Any, Dict, String

from bokehmol.config import settings
from bokehmol.models.base_formatter import BaseFormatter


class SmilesDrawerFormatter(BaseFormatter):
    __implementation__ = "smilesdrawer_formatter.ts"
    __javascript__ = settings.smilesdrawer_src

    theme = String(
        default="light",
        help="light, dark, oldschool, solarized, solarized-dark, matrix, github, carbon, cyberpunk, gruvbox, gruvbox-dark",
    )
    background_colour = String(default="transparent")
    mol_options = Dict(
        String, Any, default={}, help="https://smilesdrawer.surge.sh/playground.html"
    )
    reaction_options = Dict(
        String, Any, default={}, help="https://smilesdrawer.surge.sh/playground.html"
    )


if __name__ == "__main__":
    from bokeh.models import ColumnDataSource, HoverTool
    from bokeh.plotting import figure, show

    source = ColumnDataSource(
        data=dict(x=[1, 8, 3], y=[2, 5, 7], SMILES=["CCO", "c1ccccc1", "[n]"])
    )

    plot = figure(
        x_range=(0, 10),
        y_range=(0, 10),
        title="SmilesDrawer Hover",
        background_fill_color="#efefef",
        tools="",
    )

    hover_tooltip = """
    <div style="font-size: 14px;">
        <div style="position: relative; left=0;">@SMILES{custom}</div>
        <div style="position: relative;">
            <span style="font-size: 10px; word-wrap: break-word;">@SMILES</span>
        </div>
    </div>"""

    plot.add_tools(
        HoverTool(
            tooltips=hover_tooltip,
            formatters={
                "@SMILES": SmilesDrawerFormatter(
                    theme="gruvbox-dark",
                    background_colour="grey",
                    mol_options={"bondThickness": 4},
                )
            },
        )
    )

    plot.circle("x", "y", size=10, line_width=0, fill_color="red", source=source)

    show(plot)



from typing import ClassVar

from bokeh.resources import Resources

from bokehmol.config import settings


class Hook:
    """Class inherited by all models for multiple purposes:

    - avoid having to delete the `__implementation__` attribute outside of
      prototyping
    - register the models with the correct name on the JS side
    - also includes a classmethod that patches the bokeh class that handles
      including the JS dependencies in the final document.
    """

    _resolve: ClassVar = None

    def __init_subclass__(cls, **kwargs) -> None:
        if hasattr(cls, "__implementation__"):
            if settings.dev_mode:
                pass
            else:
                del cls.__implementation__
                cls.__qualified_model__ = f"{cls.__module__}.{cls.__name__}"
        super().__init_subclass__(**kwargs)

    @classmethod
    def enable(cls):
        cls._resolve = Resources._resolve

        def patched_resolve(self, *args, **kwargs):
            kind = args[0] if args else kwargs["kind"]
            files, raw, hashes = cls._resolve(self, kind)
            bokehmol_min_js = settings.bokehmol_js
            if "{" in bokehmol_min_js:
                raw.append(bokehmol_min_js)
            return files, raw, hashes

        Resources._resolve = patched_resolve

    @classmethod
    def restore(cls):
        Resources._resolve = cls._resolve



from bokehmol import hover
from bokehmol.config import settings
from bokehmol.hover import register_alias
from bokehmol.utils import Hook as _hook

if not settings.dev_mode:
    _hook.enable()

del _hook



from bokehmol.models.base_hover import BaseHover
from bokehmol.models.smilesdrawer_formatter import SmilesDrawerFormatter


class SmilesDrawerHover(BaseHover, SmilesDrawerFormatter):
    __implementation__ = "smilesdrawer_hover.ts"



from bokeh.core.properties import Override, String
from bokeh.models import HoverTool

from bokehmol.models.base_formatter import BaseFormatter


class BaseHover(BaseFormatter, HoverTool):
    __implementation__ = "base_hover.ts"

    smiles_column = String(default="SMILES", help="Column containing SMILES string")
    tooltips = Override(default=[])



import importlib.resources
import os
import warnings
from dataclasses import dataclass


@dataclass()
class Settings:
    rdkitjs_url: str = (
        "https://unpkg.com/@rdkit/rdkit@{}/Code/MinimalLib/dist/RDKit_minimal.js"
    )
    rdkitjs_version: str = "2023.3.2-1.0.0"

    smilesdrawer_url: str = (
        "https://unpkg.com/smiles-drawer@{}/dist/smiles-drawer.min.js"
    )
    smilesdrawer_version: str = "2.0.1"

    dev_mode: bool = bool(os.environ.get("BOKEHMOL", None))

    @property
    def bokehmol_js(self) -> str:
        try:
            return importlib.resources.read_text("bokehmol", "bokehmol.min.js")
        except Exception as exc:
            if self.dev_mode:
                warnings.warn(
                    "Attempting to access pre-built bokehmol extension in development "
                    "mode. This should not be necessary."
                )
                return ""
            raise IOError("Could not load pre-built bokehmol extension") from exc

    @property
    def rdkitjs_src(self) -> str:
        return self.rdkitjs_url.format(self.rdkitjs_version)

    @property
    def smilesdrawer_src(self) -> str:
        return self.smilesdrawer_url.format(self.smilesdrawer_version)


settings = Settings()
